<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\WooCommerce\FrontEnd;

use \ShipQuik\WooCommerce\ShipQuik_Shipments;
use \ShipQuik\WooCommerce\ShipQuik_ShippingRate;
use \ShipQuik\Shared\ShipQuik_ShipmentLogs;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

class ShipQuik_CheckOut
{
    /**
     * Parent object.
     *
     * @var    object
     * @access private
     * @since  1.0.0
     */
    protected $parent = null; 

    /**
     * General configuration.
     *
     * @var    object
     * @access public
     * @since  1.0.0
     */
    protected $config = null;

    public function __construct($parent)
    {
        $this->parent = $parent;
        $this->config = $parent->config;
    }

    function checkout_shipping_additional_field($rate, $index)
    {
        if (ShipQuik_ShippingRate::isShipQuik($rate) && is_checkout()) {
            $data = $rate->get_meta_data();
            $supplierInfo = json_decode($data['_supplier_data'], true);
            $customerDNI = $this->config->getValueByKey('customerFieldDNI');

            if ($supplierInfo['isTaric'] && $supplierInfo['supplier'] == 'CORREOS' && !$customerDNI) {
                woocommerce_form_field('ship_quik_customer_dni[' . $rate->get_id() . ']' , array(
                    'type'          => 'text',
                    'class'         => array('form-row-wide ship_quick_customer_dni'),
                    'label'         => __('Customer DNI/CIF/NIE', 'ship-quik'),
                    'required'      => true
                ), '');
            }
        }
    }

    public function order_shipping_to_display_shipped_via($shipped_via, $order) {
        $shippingMethods = $order->get_shipping_methods();

        $shippingMethod = [];

        // TODO: We must have only one.
        foreach ($shippingMethods as $key => $value) {
            $shippingMethod = $value;
        }
        $supplier = [];

        foreach ($shippingMethod->get_meta_data() as $meta) {
            $item = $meta->get_data();
            if ($item['key'] == 'Supplier') {
                $supplier = $item['value'];

                break;
            }
        }

        if ($supplier) {
            $shipped_via = '&nbsp;<small class="shipped_via">' . sprintf(__( 'via %s', 'woocommerce' ), $supplier) . '</small>';
        }

        return $shipped_via;
    }

    public function adminErrorNotice()
    {
        echo '<div class="notice notice-warning is-dismissible">' . $this->errorMessage . '</div>';
        $this->errorMessage = '';
    }

    public function processSendOrder($order_id, $ajax = true)
    {
        $order = wc_get_order($order_id);

        $trackingNumber    = $order->get_meta('_shipquik_tracking_number');
        $pickupNumber      = $order->get_meta('_shipquik_pickup_number');
        $pickupDatetime    = $order->get_meta('_shipquik_pickupdatetime');
        $labelError        = $order->get_meta('_shipquik_label_error');
        $shipquik_payment  = $order->get_meta('_shipquik_payment');

        $shippingMethods = $order->get_shipping_methods();

        $shippingMethod  = [];

        // TODO: We must have only one.
        foreach ($shippingMethods as $key => $value) {
            $shippingMethod = $value;
        }

        if (ShipQuik_ShippingRate::isShipQuik($shippingMethod) && !$pickupNumber && !$trackingNumber && $pickupDatetime && !$labelError) {
            $shiQuikShipments = new ShipQuik_Shipments($this->parent, $order->get_id(), $shipquik_payment['shipment']['boxes']);

            $result = $shiQuikShipments->send();

            if ($result) {
                if ($result['status'] == 200) {
                    $fileType = $result['data']['shippingOrder']['labelsMimeType'];

                    if (isset($result['data']['shippingOrder']['trackingNumber'])) {
                        $order->update_meta_data('_shipquik_tracking_number', $result['data']['shippingOrder']['trackingNumber']);
                    }
                    if (isset($result['data']['shippingOrder']['pickupRequestNumber'])) {
                        $order->update_meta_data('_shipquik_pickup_number', $result['data']['shippingOrder']['pickupRequestNumber']);
                    }

                    if (isset($result['data']['shippingOrder']['shippingLabels'])) {
                        $order->update_meta_data('shipquik_label_type', $fileType);

                        foreach ($result['data']['shippingOrder']['shippingLabels'] as $key => $value) {
                            $tmpName = SHIP_QUIK_DOCSPATH . '/' . $order->get_id() . '-' . $key .  ($fileType == 'image/png' ? '.png' : '.pdf');
                            if (!file_exists($tmpName)) {
                                $file = fopen($tmpName, 'x+');
                                fwrite($file, base64_decode($result['data']['shippingOrder']['shippingLabels'][$key]));
                                fclose($file);
                            }
                        }
                    }

                    $order->save_meta_data();

                    if ($this->config->getValueByKey('orderStatus') != 'do-nothing') {
                        $order->update_status($this->config->getValueByKey('orderStatus'));
                        $order->save();
                    }

                    if ($ajax) {
                        $response = [
                            'status' => 200,
                            'reference' => $result['data']['reference'],
                            'message' => 'Pedido procesado correctamente.',
                        ];

                        echo json_encode($response);
                    }
                }
            } else {
                $response = [
                    'status' => 200,
                    'message' => 'Ha habido un problema para solicitar la etiqueta, nos pondremos en contacto contigo.',
                ];

                $order->update_meta_data('_shipquik_label_error', "ERROR");
                $order->save();

                echo json_encode($response);
            }
        } else {
            $response = ['status' => 200, 'message' => 'PROCESSED', 'reference' => $order_id];
            echo json_encode($response);
        }

        if ($ajax) die();
    }

    public function checkout_field_process()
    {
        if (!empty($_POST['shipping_method'])) {
            if (isset($_POST['ship_quik_customer_dni'][$_POST['shipping_method'][0]])) {
                if (!$_POST['ship_quik_customer_dni'][$_POST['shipping_method'][0]]) {
                    wc_add_notice(__('Some carriers require a valid DNI/CIF/NIE for international shipments, enter it in the selected carrier.', 'ship-quik' ), 'error');
                }
            }
        }
    }

    public function checkout_field_update_order_meta($order_id)
    {
        if (!empty($_POST['shipping_method'])) {
            if (!empty($_POST['ship_quik_customer_dni'][$_POST['shipping_method'][0]])) {
                update_post_meta($order_id, '_ship_quik_customer_dni', sanitize_text_field($_POST['ship_quik_customer_dni'][$_POST['shipping_method'][0]]));
            }
        }
    }

    public function after_thankyou($order_id)
    {
        if ($order_id) {

            $order = wc_get_order($order_id);

            $shiQuikShipments = new ShipQuik_Shipments($this->parent, $order_id, []);

            $order->update_meta_data('_shipquik_payment_status', '0');

            $order->save();
        }
    }

    public function process_ajax_sendOrder()
    {
        if (!isset($_REQUEST['nonce'])) {
            exit("No naughty business please");
        }
    

        $nonce = sanitize_key($_REQUEST['nonce']);
        if (!wp_verify_nonce($nonce, "process_sendorder")) {
            exit("No naughty business please");
        }

        $key = sanitize_text_field($_REQUEST['key']);

        echo esc_attr($this->processSendOrder($key));
    }
}
